/*
license GPL

*/

#include "AURP-header.h"

namespace ns3 {

AURPHeader::AURPHeader () :
    Header (),
    jumps (1),
    m_id (0),
    destino_id(0),
    m_packetNumber (0),
		PHE(false),
		Data(false),
		auv(false)
{}

AURPHeader::AURPHeader (uint16_t jum, uint16_t orig, uint16_t destino, uint16_t packetNumber, bool phe, bool da, bool au) :
    Header (),
    jumps (jum),
    m_id (orig),
    destino_id(destino),
    m_packetNumber (packetNumber),
		PHE(phe),
		Data(da),
		auv(au)
{}

AURPHeader::~AURPHeader ()
{}


TypeId
AURPHeader::GetTypeId (void)
{
  static TypeId tid = TypeId ("ns3::AURPHeader")
    .SetParent<Header> ()
    .AddConstructor<AURPHeader> ()
  ;
  return tid;
}

TypeId
AURPHeader::GetInstanceTypeId (void) const
{
  return GetTypeId ();
}


uint32_t
AURPHeader::GetSerializedSize (void) const
{
  return sizeof(uint16_t) + sizeof (uint16_t) +sizeof (uint16_t) + sizeof (uint16_t)+ sizeof(bool)+ sizeof(bool)+ sizeof(bool);
}

void
AURPHeader::Serialize (Buffer::Iterator start) const
{
	uint8_t const* buffer;
	
	buffer = (uint8_t const*) &jumps;
	start.Write(buffer, sizeof(uint16_t));

	buffer = (uint8_t const*) &m_id;
	start.Write(buffer, sizeof (uint16_t));
	
	buffer = (uint8_t const*) &destino_id;
	start.Write(buffer, sizeof (uint16_t));

	buffer = (uint8_t const*) &m_packetNumber;
	start.Write(buffer, sizeof(uint16_t));

	buffer = (uint8_t const*) &PHE;
	start.Write(buffer, sizeof(bool));
	
	buffer = (uint8_t const*) &Data;
	start.Write(buffer, sizeof(bool));
	
	buffer = (uint8_t const*) &auv;
	start.Write(buffer, sizeof(bool));

}

uint32_t
AURPHeader::Deserialize (Buffer::Iterator start)
{
	uint8_t *buffer;
	Buffer::Iterator rbuf = start;

	buffer = (uint8_t*) &jumps;
	rbuf.Read (buffer, sizeof(uint16_t));

	buffer = (uint8_t*) &m_id;
	rbuf.Read (buffer, sizeof (uint16_t));
	
	buffer = (uint8_t*) &destino_id;
	rbuf.Read (buffer, sizeof (uint16_t));

	buffer = (uint8_t*) &m_packetNumber;
	rbuf.Read (buffer, sizeof(uint16_t));

	buffer = (uint8_t*) &PHE;
	rbuf.Read (buffer, sizeof(bool));
	
	buffer = (uint8_t*) &Data;
	rbuf.Read (buffer, sizeof(bool));
	
	buffer = (uint8_t*) &auv;
	rbuf.Read (buffer, sizeof(bool));

	return rbuf.GetDistanceFrom (start);
}

void
AURPHeader::Print (std::ostream &os) const
{
  os << "AURPHeader -> jumps= " << jumps << " ,origen= " << m_id <<" ,destino= " <<destino_id <<  " ,PHE= "<<PHE<< " ,Data= "<<Data<<" ,auv= "<< auv<<" ,packetNumber= " << m_packetNumber ; 
}

//AURPHeaderAUV
AURPHeaderAUV::AURPHeaderAUV () :
    Header (),
    m_id (0),
    destino_id(0),
    min(0),
    max(0),
		peticion(false),
		sink(false),
		ack(false)
{}

/*AURPHeaderAUV::AURPHeaderAUV (uint16_t jum, uint16_t orig, uint16_t destino, uint16_t packetNumber, bool phe, bool da, bool au) :
    Header (),
    jumps (jum),
    m_id (orig),
    destino_id(destino),
    m_packetNumber (packetNumber),
		PHE(phe),
		Data(da),
		auv(au)
{}*/

AURPHeaderAUV::~AURPHeaderAUV ()
{}


TypeId
AURPHeaderAUV::GetTypeId (void)
{
  static TypeId tid = TypeId ("ns3::AURPHeaderAUV")
    .SetParent<Header> ()
    .AddConstructor<AURPHeaderAUV> ()
  ;
  return tid;
}

TypeId
AURPHeaderAUV::GetInstanceTypeId (void) const
{
  return GetTypeId ();
}


uint32_t
AURPHeaderAUV::GetSerializedSize (void) const
{
  return sizeof(uint16_t) + sizeof (uint16_t) +sizeof (uint16_t)+sizeof (uint16_t) + sizeof (uint16_t)+ sizeof(bool)+ sizeof(bool)+sizeof(bool);
}

void
AURPHeaderAUV::Serialize (Buffer::Iterator start) const
{
	uint8_t const* buffer;

	buffer = (uint8_t const*) &m_id;
	start.Write(buffer, sizeof (uint16_t));
	
	buffer = (uint8_t const*) &destino_id;
	start.Write(buffer, sizeof (uint16_t));
	
	buffer = (uint8_t const*) &min;
	start.Write(buffer, sizeof(uint16_t));

	buffer = (uint8_t const*) &max;
	start.Write(buffer, sizeof(uint16_t));
	
	buffer = (uint8_t const*) &m_packetNumber;
	start.Write(buffer, sizeof(uint16_t));

	buffer = (uint8_t const*) &peticion;
	start.Write(buffer, sizeof(bool));
	
	buffer = (uint8_t const*) &sink;
	start.Write(buffer, sizeof(bool));
	
	buffer = (uint8_t const*) &ack;
	start.Write(buffer, sizeof(bool));

}

uint32_t
AURPHeaderAUV::Deserialize (Buffer::Iterator start)
{
	uint8_t *buffer;
	Buffer::Iterator rbuf = start;
	

	buffer = (uint8_t*) &m_id;
	rbuf.Read (buffer, sizeof (uint16_t));
	
	buffer = (uint8_t*) &destino_id;
	rbuf.Read (buffer, sizeof (uint16_t));

	buffer = (uint8_t*) &min;
	rbuf.Read (buffer, sizeof(uint16_t));
	
	buffer = (uint8_t*) &max;
	rbuf.Read (buffer, sizeof(uint16_t));
	
	buffer = (uint8_t*) &m_packetNumber;
	rbuf.Read (buffer, sizeof(uint16_t));

	buffer = (uint8_t*) &peticion;
	rbuf.Read (buffer, sizeof(bool));
	
	buffer = (uint8_t*) &sink;
	rbuf.Read (buffer, sizeof(bool));
	
	buffer = (uint8_t*) &ack;
	rbuf.Read (buffer, sizeof(bool));

	return rbuf.GetDistanceFrom (start);
}

void
AURPHeaderAUV::Print (std::ostream &os) const
{
  os << "AURPHeaderAUV -> origen= " << m_id <<" ,desino= " <<destino_id << " ,min= "<< min << " ,max= " << max <<" ,peticion= "<< peticion <<" ,sink= "<< sink <<" ,ack= "<< ack<< " ,m_packetNumber= "<< m_packetNumber ; 
}

} //namespace ns3

