#include "AURP-node.h"
#include "ns3/log.h"
#include "ns3/simulator.h"
#include "ns3/uan-header-common.h"

#include "ns3/mobility-model.h"
#include "ns3/node.h"

#include <algorithm> //list find




NS_LOG_COMPONENT_DEFINE ("AURPNode");

namespace ns3 {

/////Constructores
TypeId
DBRQueueElement::GetTypeId (void)
{
	static TypeId tid = TypeId ("ns3::DBRQueueElement")
		.SetParent <Object> ()
		.AddConstructor <AURPNode> ()
	;
	return tid;
}

TypeId
AURPpacket::GetTypeId (void)
{
	static TypeId tid = TypeId ("ns3::AURPpacket")
		.SetParent <Object> ()
		.AddConstructor <AURPNode> ()
	;
	return tid;
}		

TypeId
AURPNode::GetTypeId (void)
{
	static TypeId tid = TypeId ("ns3::AURPNode")
		.SetParent <Object> ()
		.AddConstructor <AURPNode> ()
	;
	return tid;
}		

//valores por defecto
AURPNode::AURPNode ()
{
	 tipe= 4; 
	 ms_PHE=0;
	 ms_data=0;
	 mr_PHE=0;
	 mr_data=0;
	 mr_auv=0;
	 ms_auv=0;
	 mcreate_data=0;
	 mr_solicitud=0;
	 mr_ack=0;	
	 ms_solicitud=0;
	 ms_ack=0;	
	 seq=1;
	 max=10;
	 min=5;
	 id=0;
	 GW_id=0;
	 Analisis_PHE=false;
	 time_PHE_wait= 150;
	 Activo=true;
	 jumps=0;
	 enviando=false;
	 random_wait_inicio_envios=100;
}

//declaración del medio
void
AURPNode::SetNetDevice (Ptr <NetDevice> netDevice)
{
	m_netDevice = DynamicCast <UanNetDevice> (netDevice);
}

Ptr <NetDevice>
AURPNode::GetNetDevice ()
{
	return m_netDevice;
}

void
AURPNode::ObtenerGW(Ptr<Node> node){
	//std::cout << "Obtener GW"<< std::endl;
	std::list<DBRQueueElement>::const_iterator
    lit (m_PHE.begin()),
    lend(m_PHE.end());

  
  DBRQueueElement aux= *lit;
  next_jump= aux.GetJumps();
  next_GW= aux.GetID();
  ++lit;
  for(;lit!=lend;++lit){
  	aux=*lit;
  	if(aux.GetJumps()<next_jump){
  		next_jump= aux.GetJumps();
  		next_GW= aux.GetID();
  	}
  }
  h_PHE.splice(h_PHE.end(), m_PHE); 
  jumps=next_jump;
  GW_id=next_GW;
  Analisis_PHE=false;
  //std::cout << "Escoguido: nodo "<< id << " GW "<< GW_id<< " saltos "<< jumps<< std::endl;
  Send_PHE(node);
}

//Modificación para envio de PHE por GW
void
AURPNode::Send_PHE (Ptr<Node> node)
{
	//std::cout << "Entramos en envio PHE"<< std::endl;
	if(Activo){
		AURPHeader aurpHeader;	//creamos la cabecera
		aurpHeader.SetPHE(true); //marcador de tipo de mensaje
		aurpHeader.SetOrig(id);
		aurpHeader.SetJumps(jumps+1);
		// create an empty 17 bytes packet
		Ptr<Packet> pkt = Create<Packet> (aurpHeader.GetSerializedSize());
		

		pkt-> AddHeader(aurpHeader); //Le cambias la cabecera por la cabecera creada
		// send the packet in broadcast
		Ptr<UanNetDevice> dev = node->GetDevice (0)->GetObject<UanNetDevice> ();
		dev->Send (pkt, dev->GetBroadcast (), 0);
		ms_PHE++;
	}

}

//envio de piticion a conexion a una AUV
void
AURPNode::Send_Solicitud (Ptr<Node> node, uint16_t destino)
{
	//std::cout << "Entramos en envio Solicitud"<< std::endl;
	if(Activo){
	if(!enviando){
		AURPHeaderAUV aurpHeaderAUV;	//creamos la cabecera
		aurpHeaderAUV.SetPeticion(true); //marcador de tipo de mensaje
		aurpHeaderAUV.SetOrig(id);
		aurpHeaderAUV.SetDes(destino);
		aurpHeaderAUV.SetMin(min);
		aurpHeaderAUV.SetMax(max);
		// create an empty  packet de tamaño la cabecera
		Ptr<Packet> pkt = Create<Packet> (aurpHeaderAUV.GetSerializedSize());
		

		pkt-> AddHeader(aurpHeaderAUV); //Le cambias la cabecera por la cabecera creada
		// send the packet in broadcast
		Ptr<UanNetDevice> dev = node->GetDevice (1)->GetObject<UanNetDevice> ();
		dev->Send (pkt, dev->GetBroadcast (), 0);
		ms_solicitud++;
	}}

}

//Cuando el sink quiere decirle a la auv que le envie los datos.
void
AURPNode::Send_Solicitud_enviar (Ptr<Node> node, uint16_t destino)
{
	//std::cout << "Entramos en envio Solicitud de envio"<< std::endl;
	if(Activo){
		AURPHeaderAUV aurpHeaderAUV;	//creamos la cabecera
		aurpHeaderAUV.SetSink(true); //marcador de tipo de mensaje
		aurpHeaderAUV.SetOrig(id);
		aurpHeaderAUV.SetDes(destino);
		aurpHeaderAUV.SetMin(min);
		aurpHeaderAUV.SetMax(max);
		// create an empty  packet de tamaño la cabecera
		Ptr<Packet> pkt = Create<Packet> (aurpHeaderAUV.GetSerializedSize());
		

		pkt-> AddHeader(aurpHeaderAUV); //Le cambias la cabecera por la cabecera creada
		// send the packet in broadcast
		Ptr<UanNetDevice> dev = node->GetDevice (1)->GetObject<UanNetDevice> ();
		dev->Send (pkt, dev->GetBroadcast (), 0);
		ms_sink++;
	}

}

//Enviar el mensaje de Aceptacion de la coneción a la AUV
void
AURPNode::Send_Ack (Ptr<Node> node, uint16_t destino)
{
	//std::cout << "Entramos en envio ACK"<< std::endl;
	if(Activo){
		AURPHeaderAUV aurpHeaderAUV;	//creamos la cabecera
		aurpHeaderAUV.SetAck(true); //marcador de tipo de mensaje
		aurpHeaderAUV.SetOrig(id);
		aurpHeaderAUV.SetDes(destino);
		aurpHeaderAUV.SetMin(min);
		aurpHeaderAUV.SetMax(max);
		//aurpHeaderAUV.SetPacketNumber(seq++);
		// create an empty  packet de tamaño la cabecera
		Ptr<Packet> pkt = Create<Packet> (aurpHeaderAUV.GetSerializedSize());
		

		pkt-> AddHeader(aurpHeaderAUV); //Le cambias la cabecera por la cabecera creada
		// send the packet in broadcast
		Ptr<UanNetDevice> dev = node->GetDevice (1)->GetObject<UanNetDevice> ();
		dev->Send (pkt, dev->GetBroadcast (), 0);
		ms_ack++;
	}

}

//Modificación para envio de PHE por GW
void
AURPNode::Send_AUV_prox (Ptr<Node> node)
{
	AURPHeader aurpHeader;	//creamos la cabecera
	aurpHeader.SetAuv(true); //marcador de tipo de mensaje
	aurpHeader.SetOrig(id);
  // create an empty 17 bytes packet
  Ptr<Packet> pkt = Create<Packet> (aurpHeader.GetSerializedSize());

	pkt-> AddHeader(aurpHeader); //Le cambias la cabecera por la cabecera creada
  // send the packet in broadcast
  Ptr<UanNetDevice> dev = node->GetDevice (0)->GetObject<UanNetDevice> ();
  dev->Send (pkt, dev->GetBroadcast (), 0);

	ms_auv++;
}

void AURPNode::SendData(Ptr<Node> node, Ptr <Packet> packet,uint16_t canal){
//std::cout << "Entramos en el enviador"<< std::endl;

  Ptr<UanNetDevice> dev = node->GetDevice (canal)->GetObject<UanNetDevice> ();
  dev->Send (packet, dev->GetBroadcast (), 0);
  ms_data++;
}

void AURPNode::Send_buffer(Ptr<Node> node, uint16_t destino, uint16_t canal){
//std::cout << "Entramos en el emisor buffer"<< std::endl;
	if(!enviando){
		if (m_enviar.size()!=0){
			enviando=true;
			std::list<AURPpacket>::const_iterator
				lit (m_enviar.begin()),
				lend(m_enviar.end());
		  
		
			//std::cout << "***Buffer size "<<m_enviar.size() << " del nodo "<< id << " Tiempo: "<< Simulator::Now() << std::endl;
			Send_buffer_loop (node, destino, canal, lit, lend);
		}
		/*AURPHeader aurpHeader;	//creamos la cabecera
		Ptr<Packet> pkt=	Create<Packet>();
		 const Ptr<Packet> pktaux;
	 	AURPpacket aurppacket=AURPpacket(pkt);
	 	
		while(lit!=lend){
			
			aurppacket=*lit;
			pkt= 	Create<Packet>(aurppacket.GetPacket());											//obtener el paquete y modifiar el destino para enviar
			pkt->RemoveHeader(aurpHeader);
			aurpHeader.SetDes(destino);
			pkt-> AddHeader(aurpHeader);
			SendData(node,pkt,canal);
			//std::cout << "enviando dato"<< std::endl
			std::cout << "Buffer size "<<m_enviar.size() << " del nodo "<< id << " Tiempo: "<< Simulator::Now() << std::endl;
			//WaitPoint::Wait ( Seconds (1));
			lit++;
		}*/
		//h_enviados.splice(h_enviados.end(), m_enviar);  	
		//enviando=false;
  }//else //std::cout << "buffer ya se esta enviando "<< id<< std::endl;
}

void AURPNode::Send_buffer_loop(Ptr<Node> node, uint16_t destino, uint16_t canal, std::list<AURPpacket>::const_iterator lit,
								 std::list<AURPpacket>::const_iterator lend ){
	if(lit!=m_enviar.end()){
			AURPHeader aurpHeader;	//creamos la cabecera
			Ptr<Packet> pkt=	Create<Packet>();
			AURPpacket aurppacket=AURPpacket(pkt);
			
			aurppacket=*lit;
			pkt= 	Create<Packet>(aurppacket.GetPacket());											//obtener el paquete y modifiar el destino para enviar
			pkt->RemoveHeader(aurpHeader);
			aurpHeader.SetDes(destino);
			pkt-> AddHeader(aurpHeader);
			SendData(node,pkt,canal);
			//std::cout << "enviando dato"<< std::endl
			//std::cout << "Buffer size "<<m_enviar.size() << " del nodo "<< id << " Tiempo: "<< Simulator::Now() << std::endl;
			//WaitPoint::Wait ( Seconds (1));
			
			/*std::list<AURPpacket>::const_iterator begin (m_enviar.begin());
			
			h_enviados.splice(h_enviados.end(), m_enviar,begin,lit);*/
			lit++;
			
			//ns3::Time ti = Time(10s);
			Simulator::Schedule ( Seconds (0.05), &AURPNode::Send_buffer_loop,
															 this, 
															 node, destino, canal, lit, lend);
	}else{
		
		h_enviados.splice(h_enviados.end(), m_enviar);  	
		enviando=false;
		
	}
}

void AURPNode::Create_Data_Pack (Ptr<Node> node, uint16_t size ){
	//std::cout << "Entramos en el creador"<< std::endl;
	AURPHeader aurpHeader;	//creamos la cabecera
	aurpHeader.SetData(true); //marcador de tipo de mensaje
	aurpHeader.SetOrig(id);
			//std::cout << "-------------Gw_id: "<< GW_id<< std::endl;
	aurpHeader.SetDes(GW_id);
				//std::cout << "-------------aurpHeader.GetDes() "<<aurpHeader.GetDes()<< std::endl;
	aurpHeader.SetPacketNumber(seq++);
	
	Ptr<Packet> pkt = Create<Packet> (aurpHeader.GetSerializedSize()+ size);

	pkt-> AddHeader(aurpHeader); //Le cambias la cabecera por la cabecera creada
	m_enviar.push_front(AURPpacket(pkt));
	mcreate_data++;
}

void AURPNode::Gestor_Data(Ptr<Node> node, uint16_t size, int time){
	if(Activo){
	//std::cout << "Entramos en el gestor"<< std::endl;
	 Create_Data_Pack(node,size);
	 if(id!=GW_id)
	 		Send_buffer(node, GW_id, 0);
	 //std::cout << "finalizar"<< std::endl;
	 if(time >0){
	 	Simulator::Schedule (Seconds (time),
                       &AURPNode::Gestor_Data,
                       this,
                       node,size,time);
   }
  }
}
//////Recepción
bool
AURPNode::ReceiveSink ( Ptr <const Packet> packet, Ptr<Node> node)
{
//std::cout << "Entramos en Recepción Sink"<< std::endl;
if(Activo){
	mrec++; mrec0++;
	AURPHeader aurpHeader;	//creamos la cabecera

  packet->PeekHeader (aurpHeader);
  
  if (aurpHeader.GetPHE()){
  				NS_LOG_FUNCTION("Sumidero recibe paquete "<< aurpHeader.GetOrig() << aurpHeader.GetPacketNumber() << " de PHE.");
  				mr_PHE++;
  	return true;
  	
  	
  }else{
  	if (aurpHeader.GetData()){
  		if(aurpHeader.GetDes()==id){
  					NS_LOG_FUNCTION("Sumidero recibe paquete "<< aurpHeader.GetOrig() << aurpHeader.GetPacketNumber() << " de Datos.");
  					mr_data++;
  		}else{
  		  					NS_LOG_FUNCTION("Sumidero recibe paquete "<< aurpHeader.GetOrig() << aurpHeader.GetPacketNumber() << " de Datos que no era para el era para " << aurpHeader.GetDes() <<" .");
  		  }
  					return true;
  	}else{
  		if(aurpHeader.GetAuv()){
  			NS_LOG_FUNCTION("Sumidero recibe paquete "<< aurpHeader.GetOrig() << aurpHeader.GetPacketNumber() << " de AUV cercana.");
  			Simulator::Schedule (Seconds(rand()%random_wait_inicio_envios),&AURPNode::Send_Solicitud_enviar, this, node, aurpHeader.GetOrig());
  			//envias una solicitud de conexion esperando un tiempo aleatorio
  			//Send_Solicitud_enviar(node, aurpHeader.GetOrig());
  			mr_auv++;
  			return true;
  		}
  	}
  
  
  }
}
  return false;
}


bool
AURPNode::ReceiveAUV ( Ptr <const Packet> packet, Ptr<Node> node)
{
//std::cout << "Entramos en Recepción AUV"<< std::endl;
if(Activo){
	mrec++; mrec0++;
	AURPHeader aurpHeader;	//creamos la cabecera

  packet->PeekHeader (aurpHeader);
  
  if (aurpHeader.GetPHE()){
  				NS_LOG_FUNCTION("AUV recibe paquete "<< aurpHeader.GetOrig() << aurpHeader.GetPacketNumber() << " de PHE.");
  				mr_PHE++;
  	return true;
  	
  	
  }else{
  	if (aurpHeader.GetData()){
  		if(aurpHeader.GetDes()==id){
  					NS_LOG_FUNCTION("AUV recibe paquete "<< aurpHeader.GetOrig() << aurpHeader.GetPacketNumber() << " de Datos.");
  					mr_data++;
  		}else{
  		  					NS_LOG_FUNCTION("AUV recibe paquete "<< aurpHeader.GetOrig() << aurpHeader.GetPacketNumber() << " de Datos que no era para el era para " << aurpHeader.GetDes() <<" .");
  		  }
  					return true;
  	}else{
  		if(aurpHeader.GetAuv()){
  			NS_LOG_FUNCTION("AUV recibe paquete "<< aurpHeader.GetOrig() << aurpHeader.GetPacketNumber() << " de AUV cercana.");
  			mr_auv++;
  			return true;
  		}
  	}
  
  
  }
}
  return false;
}

bool
AURPNode::ReceiveGW ( Ptr <const Packet> packet,Ptr<Node> node)
{
//std::cout << "Entramos en Recepción"<< std::endl;
if(Activo){
	mrec++; mrec0++;
	AURPHeader aurpHeader;	//creamos la cabecera

  packet->PeekHeader (aurpHeader);
  
  if (aurpHeader.GetPHE()){
  	NS_LOG_FUNCTION("GW "<< id << " recibe paquete "<< aurpHeader.GetOrig() << aurpHeader.GetPacketNumber() << " de PHE.");
		mr_PHE++;
  	
  	return true;
  	
  	
  }else{
    //std::cout << "data= "<< aurpHeader.GetData()<< std::endl;
  	if (aurpHeader.GetData()){
  	  	//std::cout << "recivido dato"<< std::endl;
  	  	//std::cout << "aurpHeader.GetDes() "<<aurpHeader.GetDes()<< std::endl;
  		if(aurpHeader.GetDes()==id){
  		  			//std::cout << "recivido dato para mi"<< std::endl;
  					NS_LOG_FUNCTION("GW "<< id << " recibe paquete "<< aurpHeader.GetOrig() << aurpHeader.GetPacketNumber() << " de Datos.");
  					m_enviar.push_back(AURPpacket(packet));
  					mr_data++;
  		}else{
  			NS_LOG_FUNCTION("GW "<< id << " recibe paquete "<< aurpHeader.GetOrig() << aurpHeader.GetPacketNumber() << " de Datos.que no era para el era para " << aurpHeader.GetDes() <<" .");
  		}
  					return true;
  	}else{
  		if(aurpHeader.GetAuv()){
  			NS_LOG_FUNCTION("GW "<< id << " recibe paquete "<< aurpHeader.GetOrig() << aurpHeader.GetPacketNumber() << " de AUV cercana.");
  			Simulator::Schedule (Seconds(rand()%random_wait_inicio_envios),&AURPNode::Send_Solicitud, this, node, aurpHeader.GetOrig());
  			//envias una solicitud de conexion esperando un tiempo aleatorio
  			//Send_Solicitud(node, aurpHeader.GetOrig());
  			mr_auv++;
  			return true;
  		}
  	}
  
  
  }
}
  return false;
}  
bool
AURPNode::ReceiveSensor ( Ptr <const Packet> packet, Ptr<Node> node)
{
if(Activo){
	mrec++; mrec0++;
	AURPHeader aurpHeader;	//creamos la cabecera

  packet->PeekHeader (aurpHeader);
  
  if (aurpHeader.GetPHE()){
  				NS_LOG_FUNCTION("Sensor "<< id << "  recibe paquete"<< aurpHeader.GetOrig() << aurpHeader.GetPacketNumber() << " de PHE.");
  				  	m_PHE.push_back(DBRQueueElement(aurpHeader.GetOrig(), aurpHeader.GetJumps()));
  	mr_PHE++;
  	if(!Analisis_PHE){
  		Simulator::Schedule (Seconds (time_PHE_wait), &AURPNode::ObtenerGW, this, node);
  		Analisis_PHE=true;
  	}
  	return true;
  	
  	
  }else{
  	if (aurpHeader.GetData()){
  		if(aurpHeader.GetDes()==id){
  					NS_LOG_FUNCTION("Sensor "<< id << "  recibe paquete"<< aurpHeader.GetOrig() << aurpHeader.GetPacketNumber() << " de Datos.");
  					m_enviar.push_back(AURPpacket(packet));
  					mr_data++;
  					
  		}else{
  			NS_LOG_FUNCTION("Sensor "<< id << "  recibe paquete"<< aurpHeader.GetOrig() << aurpHeader.GetPacketNumber() << " de Datos que no era para el era para " << aurpHeader.GetDes() <<" .");
  		}
  					return true;
  	}else{
  		if(aurpHeader.GetAuv()){
  			NS_LOG_FUNCTION("Sensor "<< id << "  recibe paquete"<< aurpHeader.GetOrig() << aurpHeader.GetPacketNumber() << " de AUV cercana.");
  			mr_auv++;
  			return true;
  		}
  	}
  
  
  }
}
  return false;
}

//funciones de recepción par amensajes especiales con el submarino
bool
AURPNode::ReceiveSink_submarino ( Ptr <const Packet> packet, Ptr<Node> node)
{ 
	if(Activo){
		mrec++;
		mrec1++;
		
		if(packet->GetSerializedSize()>118){
			AURPHeaderAUV aurpHeaderAUV;
		
			packet->PeekHeader (aurpHeaderAUV);
			
			if(aurpHeaderAUV.GetDes()==id){
				if(aurpHeaderAUV.GetPeticion()){
						NS_LOG_FUNCTION("Sink "<< id << "  recibe paquete"<< aurpHeaderAUV.GetOrig() << " de Peticion de conexion.");
						mr_solicitud++;
						return true;
				}else{
					if(aurpHeaderAUV.GetSink()){
							NS_LOG_FUNCTION("Sink "<< id << "  recibe paquete"<< aurpHeaderAUV.GetOrig() << " de Peticion de conexion por parte de Sink.");
							mr_sink++;
							return true;
				}else{
					if(aurpHeaderAUV.GetAck()){
						NS_LOG_FUNCTION("Sink "<< id << "  recibe paquete"<< aurpHeaderAUV.GetOrig()  << " de ack de conexion.");
						mr_ack++;
						return true;
					}
				}}
		}}else{
			AURPHeader aurpHeader;
		
			packet->PeekHeader (aurpHeader);
			if (aurpHeader.GetData()){
  			if(aurpHeader.GetDes()==id){
  				NS_LOG_FUNCTION("Sumidero recibe paquete "<< aurpHeader.GetOrig() << aurpHeader.GetPacketNumber() << " de Datos.");
  				mr_data++;
  				return true;
  			}
  		}	
		}
	}
	return false;
}


bool
AURPNode::ReceiveAUV_submarino ( Ptr <const Packet> packet, Ptr<Node> node)
{ 
	if(Activo){
		mrec++;
		mrec1++;
		
		if(packet->GetSerializedSize()>118){
			AURPHeaderAUV aurpHeaderAUV;
		
			packet->PeekHeader (aurpHeaderAUV);
			
			if(aurpHeaderAUV.GetDes()==id){
				if(aurpHeaderAUV.GetPeticion()){
						NS_LOG_FUNCTION("AUV "<< id << "  recibe paquete"<< aurpHeaderAUV.GetOrig() << " de Peticion de conexion.");
						if(aurpHeaderAUV.GetMax()==max ){
							if(aurpHeaderAUV.GetMin()==min ){
								Send_Ack(node, aurpHeaderAUV.GetOrig());
							}
						}
						mr_solicitud++;
						return true;
				}else{
					if(aurpHeaderAUV.GetSink()){
							NS_LOG_FUNCTION("AUV "<< id << "  recibe paquete"<< aurpHeaderAUV.GetOrig() << " de Peticion de conexion por parte de Sink.");
							if(aurpHeaderAUV.GetMax()==max ){
								if(aurpHeaderAUV.GetMin()==min ){
								//std::cout << "enviar buffer dato"<< std::endl;
									Send_buffer(node, aurpHeaderAUV.GetOrig(), 1);
									mr_sink++;
								}
							}
							//mr_sink++;
							return true;
				}else{
					if(aurpHeaderAUV.GetAck()){
						NS_LOG_FUNCTION("AUV "<< id << "  recibe paquete"<< aurpHeaderAUV.GetOrig()  << " de ack de conexion.");
						mr_ack++;
						return true;
					}
				}}
		}}else{
			AURPHeader aurpHeader;
		
			packet->PeekHeader (aurpHeader);
			if (aurpHeader.GetData()){
  			if(aurpHeader.GetDes()==id){
  				//std::cout << "recivido dato para AUV"<< std::endl;
  				NS_LOG_FUNCTION("Sumidero recibe paquete "<< aurpHeader.GetOrig() << aurpHeader.GetPacketNumber() << " de Datos.");
  				m_enviar.push_back(AURPpacket(packet));
  				mr_data++;
  				return true;
  			}
  		}	
		}
	}
	return false;
}


bool
AURPNode::ReceiveGW_submarino ( Ptr <const Packet> packet, Ptr<Node> node)
{ 
	if(Activo){
		mrec++;
		mrec1++;
		
		if(packet->GetSerializedSize()>118){
			AURPHeaderAUV aurpHeaderAUV;
		
			packet->PeekHeader (aurpHeaderAUV);
			if(aurpHeaderAUV.GetDes()==id){
				if(aurpHeaderAUV.GetPeticion()){
						NS_LOG_FUNCTION("GW "<< id << "  recibe paquete"<< aurpHeaderAUV.GetOrig() << " de Peticion de conexion.");
						mr_solicitud++;
						return true;
				}else{
					if(aurpHeaderAUV.GetSink()){
							NS_LOG_FUNCTION("GW "<< id << "  recibe paquete"<< aurpHeaderAUV.GetOrig() << " de Peticion de conexion por parte de Sink.");
							mr_sink++;
							return true;
				}else{
					if(aurpHeaderAUV.GetAck()){
						NS_LOG_FUNCTION("GW "<< id << "  recibe paquete"<< aurpHeaderAUV.GetOrig()  << " de ack de conexion.");
						Send_buffer(node, aurpHeaderAUV.GetOrig(), 1);
						mr_ack++;
						return true;
					}
				}}
		}}else{
			AURPHeader aurpHeader;
		
			packet->PeekHeader (aurpHeader);
			if (aurpHeader.GetData()){
  			if(aurpHeader.GetDes()==id){
  				NS_LOG_FUNCTION("GW recibe paquete "<< aurpHeader.GetOrig()<<"con numero de pack: " << aurpHeader.GetPacketNumber() << " de Datos.");
  				mr_data_not++;
  				return true;
  			}
  		}	
		}
	}
	return false;
}

;} //namespace ns3

