/*
license GPL
author danim1987@gmail.com
*/


#ifndef AURP_NODE_H
#define AURP_NODE_H

#include <list>

#include "ns3/event-id.h"
#include "ns3/vector.h"
#include "ns3/packet.h"
#include "ns3/uan-net-device.h"
#include "ns3/uan-address.h"
#include "ns3/nstime.h"

#include "AURP-header.h"

#include "ns3/waypoint-mobility-model.h"


namespace ns3 {


class DBRQueueElement : public Object
// elemento en la cola de pendientes
{

public:
	uint16_t node_id, Jumps;
	EventId event; //evento asociado para el envio
	
	uint16_t GetJumps(){return Jumps;}
	uint16_t GetID(){return node_id;}

	static TypeId GetTypeId (void);

	DBRQueueElement(uint16_t a, uint16_t n, EventId e) {node_id = a; Jumps = n; event = e;}
	DBRQueueElement(uint16_t a, uint16_t n) {node_id = a; Jumps = n;}
	EventId GetEventID() {return event;}

        bool operator== (const DBRQueueElement &other) const {
        	if (node_id == other.node_id && Jumps == other.Jumps)
			return true;
		return false;
  	}
}; //DBRQueueElement

class AURPpacket : public Object
{
private:
	//Packet paquete;
public:
	Packet paquete;
		static TypeId GetTypeId (void);

	//AURPpacket();
	AURPpacket(Ptr <const Packet> packet){paquete=*packet;}
	const Packet GetPacket(){return paquete;}
};
/*
class DBRSentElement : public Object
//para lista de elementos enviados
{

public:
	uint16_t node_id, num;
	Time t; //cuando entro en la lista, usado para calcular retardos

	static TypeId GetTypeId (void);

	DBRSentElement(uint16_t a, uint16_t n, Time e) {node_id = a; num = n; t = e;}
	DBRSentElement(uint16_t a, uint16_t n) {node_id = a; num = n;}
	Time GetTime() {return t;}

        bool operator== (const DBRSentElement &other) const {
        	if (node_id == other.node_id && num == other.num)
			return true;
		return false;
  	}
}; //DBRSentElement
*/
///////////////////////////////////////////////////////////////////////////////////

class AURPNode : public Object
{

private:

	uint16_t id; //id del nodo, usado cabecera
	uint16_t GW_id; //id de la puerta de enlace
	uint16_t jumps; //saltos necesarios atrabes de esa puerta de enlace necesario para el mensaje PHE
	uint64_t seq; //siguiente numero secuencia
	
	int tipe; //Tipo de nodo sumidero, AUV , GW o Usensor
	int mrec, mrec0,mrec1;
	int ms_PHE;
	int ms_data;
	int mr_PHE;
	int mr_data;
	int mr_auv, ms_auv;
	int mcreate_data;
	int mr_solicitud;
	int mr_sink;
	int mr_ack;	
	int ms_solicitud;
	int ms_sink;
	int ms_ack;	
	int mr_data_not;

	uint16_t max, min; //limites de frecuencias para el envio con AUV
	
	//semaforos
	bool Analisis_PHE;		//no analizar los veces la misma lista de PHE
	bool enviando; 				//Enviar dos veces la msima lista 
	
	
	int time_PHE_wait;
	int random_wait_inicio_envios; //tiempo maximo que puede durar una comunicación con la AUV
	
	Ptr <UanNetDevice> m_netDevice;
	
	bool Activo; //cuando el nodo se queda sin bateria se desactiba.

		//lista con los paquetes y su tiempo de llegada
	std::list <DBRQueueElement> m_PacketQueue;
	
	//lista de mensajes a enviar al siguiente.
	std::list < AURPpacket> m_enviar;
	
	//Historico de mensajes enviados.
	std::list < AURPpacket> h_enviados;

	//Lista de mensajes PHE por analizar.
	std::list <DBRQueueElement> m_PHE;
	
	//Historico de mensajes PHE analizados.
	std::list <DBRQueueElement> h_PHE;
	
	//metodo que obtiene el GW con la lista de mensajes PHE
	void ObtenerGW(Ptr<Node> node);
	
	uint16_t next_jump, next_GW; 
	//variables auxiliares para obtencionGW
	

	
public:

////////////////AURP///////////////////////////////////////////////////////////
	//Es la función que genera la trayectoria de la auv con forma eliptica
	//Los parametros son, el eje mayor, el eje menor, el angulo de einclinación
	//con respecto a el eje de cordenadas, a que profundidad genera le elipse
	// y el modelo de profundidad.
	//no devuelve nada pero podría retornar el tiempo que tarda en dar el recorrido 
	//void trayectoria_eliptica (double a, double b, double angu_despla, double z, Ptr<WaypointMobilityModel> mob);
	//devuelve el tiempo que tarda en hacer la elipse
	//double Gettiempo_elipse(){return tiempo_elipse;}
//////////////AUV//////////////////////////
	AURPNode();
	void SetNetDevice (Ptr <NetDevice> netDevice);
	Ptr <NetDevice> GetNetDevice ();
	static TypeId GetTypeId (void);
	
	void SetID(uint16_t i) {id = i;}
	uint16_t GetID() {return id;}
	void SetGW_ID(uint16_t i) {GW_id = i;}
	uint16_t GetGW_ID() {return GW_id;}
	uint16_t GetJumps() {return jumps;}
	uint16_t GetSeq() {return seq;}
	void SetJumps(uint16_t i) {jumps = i;}
	
	void SetTime_PHE(int tim){time_PHE_wait=tim;}
	int GetTime_PHE(){return time_PHE_wait;}
	void SetRandomWaitInicioEnvios(int tim){random_wait_inicio_envios=tim;}
	int GetRandomWaitInicioEnvios(){return random_wait_inicio_envios;}
	
	int GetMrec(){return mrec;}
	int GetMsPHE(){return ms_PHE;}
	int GetMrPHE(){return mr_PHE;}
	int GetMsdata(){return ms_data;}
	int GetMrdata(){return mr_data;}
	int GetMsauv(){return ms_auv;}
	int GetMrauv(){return mr_auv;}
	int GetMCreate_Data(){return mcreate_data;}
	int GetMssolicitud(){return ms_solicitud;}
	int GetMrsolicitud(){return mr_solicitud;}
	int GetMssink(){return ms_sink;}
	int GetMrsink(){return mr_sink;}
	int GetMsack(){return ms_ack;}
	int GetMrack(){return mr_ack;}
	
	bool GetActivo(){return Activo;}
	void SetActivo(bool a){Activo=a;}
	
	int GetTipe(){return tipe;}
	void SetTipe(int t){tipe=t;}
	
	void SetMin(uint16_t i) {min = i;}
	uint16_t GetMin() {return min;}
	void SetMax(uint16_t i) {max = i;}
	uint16_t GetMax() {return max;}
	
//////mensajes
	//bool SendData(Ptr<Node> node, Ptr <Packet> packet);
	void Send_PHE(Ptr<Node> node);
	//Crea y envia un mensaje PHE
	void Send_Solicitud(Ptr<Node> node, uint16_t destino);
	//Crea y envia un mensaje de solicitud
	void Send_Solicitud_enviar (Ptr<Node> node, uint16_t destino);
	//Crea y envia un mensaje de solicitud para que le envien los datos.
	void Send_Ack(Ptr<Node> node, uint16_t destino);
	//Crea y envia un mensaje de aceptación de solicitud con la dirección del que pide la peticion
	void Send_AUV_prox (Ptr<Node> node);
	//Crea y envia un mensaje AUV
	void SendData(Ptr<Node> node, Ptr <Packet> packet,uint16_t canal);
	//Envia el mensaje que se le pasa
	void Send_buffer(Ptr<Node> node, uint16_t destino, uint16_t canal);
	//Envia todos los mensajes que estén en el buffer
	void Send_buffer_loop(Ptr<Node> node, uint16_t destino, uint16_t canal, std::_List_const_iterator<ns3::AURPpacket> lit,
								 std::_List_const_iterator<ns3::AURPpacket> lend );
								 //no utilizable
	void Create_Data_Pack (Ptr<Node> node, uint16_t size ); 
	//Crea un packete de datos y lo coloca en el buffer de envio 
	//recibe como entrada el nodo que envia y el tamaño de los datos(datos basura)
	void Gestor_Data(Ptr<Node> node, uint16_t size, int time);
	//Crea un pack de datos y envia todos los datos que tenga para enviar
	//recibe el nodo de envio, el tamaño de packete de datos y 
	//cada cuanto segundos se reejecuta (bucle) con -1 no bucle 
	
	uint16_t Get_Num_Pack_wait_to_send (){return m_enviar.size();}
	uint16_t Get_Num_PHE_wait (){return m_PHE.size();}

	//bool Receive (Ptr <NetDevice> device, Ptr <const Packet> packet, uint16_t protocolNumber, const Address& src);

/////recibir
//gestionan la recepción de mensajes para cada tipo de nodo
bool ReceiveSink ( Ptr <const Packet> packet, Ptr<Node> node);
bool ReceiveAUV ( Ptr <const Packet> packet, Ptr<Node> node);
bool ReceiveGW ( Ptr <const Packet> packet, Ptr<Node> node);
bool ReceiveSensor ( Ptr <const Packet> packet, Ptr<Node> node);

bool ReceiveSink_submarino ( Ptr <const Packet> packet, Ptr<Node> node);
bool ReceiveAUV_submarino ( Ptr <const Packet> packet, Ptr<Node> node);
bool ReceiveGW_submarino ( Ptr <const Packet> packet, Ptr<Node> node);



}; //class AURPNode
} //namespace ns3

#endif //AURP_NODE_H

